<?php
   namespace Financeiro\Logic\ContasPagar\BoletoPagar;

   use \Fiscal\Entities\Boletpagar;
   use \Fiscal\Entities\Contapagar;
   use \Financeiro\Entities\Ajbolpag;
   use \Financeiro\Repositories\Ajbolpag as Ajustes;
   use \Geral\Util\Ambiente;
   use \Geral\Util\DateUtil;
   use \Setforms2\Shared\Registry;
   use \Financeiro\Logic\ContasPagar\BoletoPagar\Exception\Exception;
   use \Geral\Util\NovoCodigo;

   class BoletoPagar{


       /************************************** GERAL **************************************/


       /**
        * Retorna o valor do boleto.
        * @param $boleto array
        * @return float
        */
       public static function buscaValorBoleto($boleto){
           $object = self::buscaBoleto($boleto);
           if($object){
               return $object->getVrfinalbpag();
           }
           return 0.0;
       }

       /**
        * Retorna o boleto a partir do CDFILIAL/NRLANBOLPAG/NRORG, se não encontrar retorna NULL
        * @param $boleto array
        * @return BOLETPAGAR|null
        */
       private static function buscaBoleto($boleto){
           $object = Boletpagar::getObjectByWhere("CDFILIAL = :P_CDFILIAL AND NRLANBOLPAG = :P_NRLANBOLPAG AND NRORG = :P_NRORG ",$boleto);
           if($object)
               return $object;
           else
               return null;
       }

       /**
        * Retorna o boleto a partir CDFILIAL/NRORG/NRLANBOLPAG/CDMEPRESA, se não encontrar retorna NULL
        * @param $boleto array
        * @return Contapagar|null
        */
       private static function buscaContaPorBoleto($boleto){
           $where = array(
               "P_CDEMPRESA" => $boleto['CDEMPRESA'],
               "P_CDFILIBPAG" => $boleto['CDFILIAL'],
               'P_NRORG' => $boleto['NRORG'],
               'P_NRLANBOLPAG' => $boleto['NRLANBOLPAG']
           );
           $objects = Contapagar::getObjectsByWhere("CDEMPRESA = :P_CDEMPRESA AND CDFILIBPAG = :P_CDFILIBPAG AND NRORG = :P_NRORG AND NRLANBOLPAG = :P_NRLANBOLPAG", $where);
           if ($objects)
               return $objects;
           else
               return null;
       }

       /**
        * Busca os valores definidor por fornecedor e filial.
        * @param $ajustes array
        * @return Ajbolpag|null
        */
       private static function buscaAjustesporboleto($ajustes){
           $object = Ajbolpag::getObjectsByWhere('CDFILIAL = :P_CDFILIAL AND NRLANBOLPAG = :P_NRLANBOLPAG AND NRORG = :P_NRORG',$ajustes);
           if($object){
               return ($object);
           }
           return NULL;
       }

       /**
        * Insere um novo boleto no banco de dados
        * @param $filial
        * @param $nrorg
        * @param $cdforneced
        * @param $cdempresa
        * @param $valor
        * @param $multa
        * @param $mora
        * @param $dataVencimento
        * @param $contaCorrente
        * @param $cdbancbolp
        * @param $tpbaixa
        * @param $cdagenbolp
        * @param $Cdccorbolp
        * @return Boletpagar
        */
       private  static function criarBoleto($filial,$nrorg, $cdforneced, $cdempresa, $valor, $multa, $mora, $dataVencimento,$contaCorrente, $cdbancbolp, $tpbaixa, $cdagenbolp, $Cdccorbolp, $idbpagreal = 'N' ){

           try {// RECUPERA VALORES
               $connect = Registry::getEntityManager();
               $novoCodigo=Novocodigo::geraCodigo($nrorg,'BOLETPAGAR'.$filial);
               $valorMulta=$valor*($multa/100);
               $valorMora=$valor*($mora/100);
               $valorTotal=$valor+$valorMora+$valorMulta;
               if(strlen($dataVencimento)>10)
                    $dataVencimento = DateUtil::truncateString($dataVencimento);
               $dataVencimento = DateUtil::truncateData(DateUtil::getDataDeString($dataVencimento));


               //CRIA BOLETO

               $novoBoleto = new Boletpagar();
               $novoBoleto->setCdfilial($filial);
               $novoBoleto->setNrlanbolpag($novoCodigo);
               $novoBoleto->setNrparcbolpag('00');
               $novoBoleto->setCdforneced($cdforneced);
               $novoBoleto->setCdempresa($cdempresa);
               $novoBoleto->setVrbolpag($valor);
               $novoBoleto->setVrpemulbpag($multa);
               $novoBoleto->setVrmultabpag($valorMulta);
               $novoBoleto->setVrpemorbpag($mora);
               $novoBoleto->setVrmordiabpag($valorMora);
               $novoBoleto->setVrfinalbpag($valorTotal);
               $novoBoleto->setDtvenbolpag($dataVencimento);
               $novoBoleto->setIdhabibpag('N');
               $novoBoleto->setIdstatbpag('A');
               $novoBoleto->setCdfilibaipag(NULL);
               $novoBoleto->setNrbaixapag(NULL);
               $novoBoleto->setIdbpagreal($idbpagreal);
               $novoBoleto->setCdreducont($contaCorrente);
               $novoBoleto->setCdbarrapag(NULL);
               $novoBoleto->setDslinhbarpag(NULL);
               $novoBoleto->setCdnosnumpag(NULL);
               $novoBoleto->setIdsitucnabp('00');
               $novoBoleto->setNrretcnabpag(NULL);
               $novoBoleto->setNrcnabpagrem(NULL);
               $novoBoleto->setNrcnabpagret(NULL);
               $novoBoleto->getCdbancbolp( $cdbancbolp);
               $novoBoleto->getCdagenbolp($cdagenbolp);
               $novoBoleto->setCdccorbolp($Cdccorbolp);
               $novoBoleto->setCdmotiparc(NULL);
               $novoBoleto->setNrorigbolpag(NULL);
               $novoBoleto->setIdtpicnabbop(NULL);
               $novoBoleto->setNrinscnabbop(NULL);
               $novoBoleto->setCdtpbaixapag($tpbaixa);
               $novoBoleto->setNrorg($nrorg);
//           $novoBoleto->setCdagencbolp(NULL);
               $novoBoleto->setIdativo('S');
               $novoBoleto->setDtinclusao(DateUtil::getDataAtual());
               $novoBoleto->setDtultatu(DateUtil::getDataAtual());
               $novoBoleto->setCdoperinclusao(Ambiente::getCdOperador());
               $novoBoleto->setCdoperultatu(Ambiente::getCdOperador());
               $novoBoleto->persist();
               $connect->flush();

               return ($novoBoleto);
           } catch (\Exception $e) {
               throw $e;
           }
       }



       /************************************* AJUSTES BOLETO **********************************/


       /**
        * Atualiza o valor do boleto apartir de ajustes
        * @param $boleto array
        * @param $valor
        * @return bool
        */
       public static function  atualizaValorBoleto($boleto, $valor){
           try {
               $connect = Registry::getEntityManager();
               $object = self::buscaBoleto($boleto);
               if ($object) {

                   $valorBoleto = $object->getVrfinalbpag();

                   $object->setVrfinalbpag($valorBoleto + $valor);
                   $object->persist();

                   $connect->flush();
                   tk_reload_data_with_filter('tab_pbi00017_000_004', 'win_pbi00017_000');
                   tk_reload_data_with_filter('tab_pbi00017_000_000', 'win_pbi00017_000');
                   return true;
               }
               return false;
           } catch (\Exception $e) {
               throw $e;
           }
       }



       /****************************** PRORROGAÇÃO DE BOLETO ********************************/



       /**
        * Altera a data de vencimento do boleto 'Dtvenbolpag' para uma data posterior
        * @param $boleto array
        * @param $vencimento
        * @return bool
        */
       public static function prorrogacaoBoleto($boleto,$vencimento){
           try {
               //Altera a data para um objeto Date
               $vencimento = DateUtil::truncateData(Dateutil::getDataDeString($vencimento));
    
               //Busca boleto
               $connect = Registry::getEntityManager();
               $object = self::buscaBoleto($boleto);

               //Se não está nulo, altera o vencimento
               if (($object != null) && ($object->getDtvenbolpag() < $vencimento)) {
                   $object->setDtvenbolpag($vencimento);
                   $object->persist();
                   $connect->flush();
                   return true;
               } else
                   return false;
           } catch (\Exception $e) {
               throw $e;
           }
       }



       /******************************** VALIDAÇÃO DE BOLETOS ***********************************/

       /**
        * Verifica se todos os boletos selecionados são do mesmo fornecedor.
        * @param $boletos array
        * @throw Exception/ @return bool
        */
       public static function validaFornecedor($boletos){
           try {
               //VERIFICA SE TODOS OS FORNECEDORES SÃO IGUAIS AO PRIMEIRO
               foreach ($boletos as $boleto) {
                   if ($boleto['CDFORNECED'] != $boletos[0]['CDFORNECED'])
                       Exception::fornecedoresDiferentes();
               }
               return true;
           } catch (\Exception $e) {
               throw $e;
           }
       }

       /**
        * Verifica se todos os boletos selecionados são da mesma empresa.
        * @param $boletos array
        * @throw Exception/ @return bool
        *
        * Utilizado apenas para testes.
        */
       public static function validaEmpresa($boletos){

           try {
               foreach ($boletos as $boleto) {
                   //VERIFICA SE TODOS OS BOLETOS SÃO DA MESMA EMPRESA
                   if ($boleto['CDEMPRESA'] != $boletos[0]['CDEMPRESA'])
                       Exception::empresasDiferentes();
               }
               return true;
           } catch (\Exception $e) {
               throw $e;
           }

       }


       /********************************* ASSOCIAÇÃO DE BOLETOS ***********************************/


       /**
        * Altera o NRLANBOLPAG do boleto na tabela CONTAPAGAR
        * @param $boleto array
        * @param $novoBoleto Boletpagar
        * @param $objetcs array Contapagar
        * @return bool
        * @throws \Exception
        * @var $objects Contapagar
        * @var $object Contapagar
        */
       private static function modificaConta($boleto,$novoBoleto,$objects){
           try {
               //CONECTA
               $connect = Registry::getEntityManager();

               //BUSCA A CONTA
               if (!$objects) {
                   $where = array(
                       "P_CDFILIBPAG" => $boleto['CDFILIAL'],
                       'P_NRORG' => $boleto['NRORG'],
                       'P_NRLANBOLPAG' => $boleto['NRLANBOLPAG']
                   );
                   $objects = Contapagar::getObjectsByWhere("CDFILIBPAG = :P_CDFILIBPAG AND NRORG = :P_NRORG AND NRLANBOLPAG = :P_NRLANBOLPAG", $where);
               }

               //ALTERA A CONTA
               if ($objects) {
                   foreach ($objects as $object) {

                       $object->setCdfilibpag($novoBoleto->getCdfilial());
                       $object->setNrlanbolpag($novoBoleto->getNrlanbolpag());

                       $object->persist();
                   }
                   $connect->flush();
                   return true;
               }
               return false;
           } catch (\Exception $e) {
               throw $e;
           }
       }

       /**
        * Verifica se há 1 ou mais contas a serem modificadas
        * @param $boletos array
        * @param $novoBoleto Boletpagar
        * @param $conta
        * @throws \Exception
        */
       public static function modificaContaBoleto($boletos, $novoBoleto, $conta = null){

           try {
               //SE EXISTIREM CONTAS, TRANSFORMA-LAS EM UM ARRAY PARA SEREMEXECUTADA NO FOR EACH, PARA EVITAR CRIAÇÃO DE NOVO MÉTODO.
               if($conta)
                   $conta = array( 0 => $conta);

               //VERIFICA SE EXISTEM UM OU MAIS CONTAS, SE HOUVEREM MAIS DE UMA, EXECUTAR O MODIFICA CONTA PARA CADA BOLETO
               if (isset($boletos[0])) {
                   foreach ($boletos as $boleto) {
                       self::modificaConta($boleto, $novoBoleto, $conta);
                   }
               } else {
                   self::modificaConta($boletos, $novoBoleto, $conta);
               }

           } catch (\Exception $e) {
               throw $e;
           }
       }

       /**
        * Cria um novo boleto a partir de vários boletos e faz a associação da conta
        * @param $boletos
        * @param $dataVencimento
        * @param $contaCorrente
        * @param $tpbaixa
        */
       public static function criaNovoBoleto($boletos, $dataVencimento, $contaCorrente, $tpbaixa){

           try { //DEFINIÇÃO DE VARIAVEIS
               $valor = 0;
               $multa = $boletos[0]['VRPEMULBPAG'];
               $mora = $boletos[0]['VRPEMORBPAG'];
               $cdbancbolp = $boletos[0]['CDBANCBOLP'];
               $cdagenbolp = ('00'); //$boletos[0]['CDAGENCBOLP']);
               $Cdccorbolp = ($boletos[0]['CDCCORBOLP']);
               $cdforneced = ($boletos[0]['CDFORNECED']);
               $cdempresa = ($boletos[0]['CDEMPRESA']);

               $nrorg = Ambiente::getNrorg();
               $filial = Ambiente::getCdFilial();
               $connect = Registry::getEntityManager();


               //SOMAR TODOS OS VALORES DE BOLETO, MULTA E MORA
               foreach ($boletos as $boleto) {
                   $valor += $boleto['VRBOLPAG'];
               }

               //INSERIR NOVO BOLETO

               return self::criarBoleto($filial, $nrorg, $cdforneced, $cdempresa, $valor, $multa, $mora, $dataVencimento, $contaCorrente, $cdbancbolp, $tpbaixa, $cdagenbolp, $Cdccorbolp, 'S');
           } catch (\Exception $e) {
               throw $e;
           }

       }

       /**
        * Apaga boletos que já foram associados
        * @param $boletos
        * @return bool
        * @throws \Exception
        */
       public static function apagaBoletosAnteriores($boletos){
           $connect = Registry::getEntityManager();
           //APAGA BOLETOS ANTERIORES

           $connect->flush();

           try {
               if (isset($boletos[0])) {

                   //APAGAR VÁRIOS BOLETOS
                   foreach ($boletos as $bol) {
                       $boleto = array(
                           'P_CDFILIAL' => $bol['CDFILIAL'],
                           'P_NRLANBOLPAG' => $bol['NRLANBOLPAG'],
                           'P_NRORG' => $bol['NRORG']
                       );
                       $object = self::buscaBoleto($boleto);

                       $object->delete();
                   }
               } else {

                   //APAGAR UNICO BOLETO
                   $boletos = array(
                       'P_CDFILIAL' => $boletos['CDFILIAL'],
                       'P_NRLANBOLPAG' => $boletos['NRLANBOLPAG'],
                       'P_NRORG' => $boletos['NRORG']
                   );
                   $object = self::buscaBoleto($boletos);
                   $object->delete();
//                   $object->persist();
               }
               $connect->flush();
               return true;
           } catch (\Exception $e) {
               throw $e;
           }
       }

       /**
        * Insere ajustes no banco de dados
        * @param $boletos
        * @param $novoBoleto Boletpagar
        */
       public static function criarAjustes($boletos, $novoBoleto){
           try {
               //DEFINIÇÃO DE VARIÁVEIS
               $connect = Registry::getEntityManager();
               $numBol = array();
               $cdfilial = array();
               $nrorg = 1; //Ambiente::getNrorg();

               //CRIA UM ARRAY PARA OS NUMEROS E AS FILIAIS DO BOLETOS
               foreach ($boletos as $boleto) {
                   $numBol[] = "'" . $boleto['NRLANBOLPAG'] . "'";
                   $cdfilial[] = "'" . $boleto['CDFILIAL'] . "'";
               }

               //TRANSFORMA O ARRAY EM STRINGS COM OS VALORES SEPARADOS POR VÍRGULA
               $numBol = implode(',', $numBol);
               $cdfilial = implode(',', $cdfilial);

               //BUSCA OS AJUSTES
               $ajustes = Ajustes::buscaAjustes($nrorg, $cdfilial, $numBol);

               //INSERE NOVO AJUSTE IGUAL AO QUE FOI ENCONTRADO NO BANCO MUDANDO A FILIAL E NUMERO DE LANÇAMENTO DO BOLETO
               foreach ($ajustes as $ajuste) {
                   $ajus = new Ajbolpag();
                   $ajus->setCdfilial($novoBoleto->getCdfilial());
                   $ajus->setNrlanbolpag($novoBoleto->getNrlanbolpag());
                   $ajus->setCdajuctpag($ajuste['CDAJUCTPAG']);
                   $ajus->setVrajbolpag($ajuste['SOMA']);
                   $ajus->setIdtpajbolpag(1);
                   $ajus->setIdorigajbpag(1);
                   $ajus->setNrorg($novoBoleto->getNrorg());
                   $ajus->setDtinclusao(DateUtil::getDataAtual());
                   $ajus->persist();
               }
               $connect->flush();
           } catch (\Exception $e) {
               throw $e;
           }
       }

       /**
        * Verifica se há um ou meis boletos para apagar os ajustes, se houver mais de um executar a rotina para cada um separadamente.
        * @param $boletos
        * @return bool
        * @throws \Exception
        */
       public static function deleteAjustes($boletos){
           try {
               //APAGA VÁRIOS BOLETOS
               if (isset($boletos[0])) {
                   foreach ($boletos as $boleto) {
                       self::deleteajustesBoleto($boleto);
                   }
                   return true;
               } else {
                   //APAGA APENAS UM BOLETO
                   self::deleteajustesBoleto($boletos);
                   return true;
               }
               return false;
           } catch (\Exception $e) {
               throw $e;
           }
       }

       /**
        * Deleta ajustes de boletos que foram associados
        * @var $objects Ajbolpag
        * @var $object Ajbolpag
        * @param $boleto Boletpagar
        * @return bool
        */
       public static function deleteajustesBoleto($boleto){
           $connect = Registry::getEntityManager();
           try {
               //Buscar ajustes
               $ajuste = array(
                   'P_NRORG' => $boleto['NRORG'],
                   'P_NRLANBOLPAG' => $boleto['NRLANBOLPAG'],
                   'P_CDFILIAL' => $boleto['CDFILIAL'],
               );
               $objects = self::buscaAjustesporboleto($ajuste);


               if ($objects) {
                   if (is_array($objects)) {
                       //APAGAR VÁRIOS AJUSTES
                       foreach ($objects as $object) {
                           $object->delete();
                           $connect->flush();
                       }
                       return true;
                   }else{
                       //APAGA APENAS UM AJUSTE
                       $objects->delete();
                       $connect->flush();

                       return true;
                   }
                   return false;
               }
           } catch (\Exception $e) {
               throw $e;
           }
       }

       /**************************************** DESASSOCIAR BOLETOS ***********************************************/

       /**
        * Desassocia os boletos em um boleto por conta
        * @param $boleto array
        * @var $contas array
        */
       private static function desassociar($boleto){
           try {
               $novoBoleto = new BoletoPagar();
               //DEFINIÇÕES DE VARIÁVEIS
               $cdfilial = $boleto['CDFILIAL'];
               $cdforneced = $boleto['CDFORNECED'];
               $cdempresa = $boleto['CDEMPRESA'];
               $nrorg = $boleto['NRORG'];
               $multa = $boleto['VRPEMULBPAG'];
               $mora = $boleto['VRPEMORBPAG'];
               $dataVencimento = $boleto['DTVENBOLPAG'];
               $cc = $boleto['CDREDUCONT'];
               $cdbancbolp = $boleto['CDBANCBOLP'];
               $tpbaixa = $boleto['CDTPBAIXAPAG'];
               $cdangenbolp = $boleto['CDAGENBOLP'];
               $cdccorbolp = $boleto['CDCCORBOLP'];


               //BUSCA CONTAS

               //CONSULTA BOLETOS
               $contas = self::buscaContaPorBoleto($boleto);

               /**
                * @var $conta Contapagar
                */
               foreach ($contas as $conta) {

                   //CRIA BOLETOS PARA CADA CONTA
                   $novoBoleto = self::criarBoleto($conta->getCdfilibpag(), $nrorg, $cdforneced, $cdempresa, $conta->getVrorigpag(), $multa, $mora, $dataVencimento, $cc, $cdbancbolp, $tpbaixa, $cdangenbolp, $cdccorbolp,'N');

                   //ASSOCIAR CADA CONTA A UM BOLETO
                   self::modificaContaBoleto($boleto, $novoBoleto, $conta);

               }

               //APAGA AJUSTES DO BOLETO ASSOCIADO
               self::deleteAjustes($boleto);

               //APAGA BOLETO ASSOCIADO
               self::apagaBoletosAnteriores($boleto);
           } catch (\Exception $e) {
               throw $e;
           }
       }

       /**
        * Verifica se a ação será executada para um ou mais boletos
        * @param $boletos
        * @return bool
        */
       public static function desassociaBoletos($boletos){
           try {

               //VERIFICA SE HÁ UM OU MAIS BOLETOS A SEREM DESASSOCIADOS
               if ($boletos) {
                   if (isset($boletos[0])) {
                       //DESASSOCIA UM GRUPO DE BOLETOS EM VÁRIAS CONTAS
                       foreach ($boletos as $boleto) {
                           self::desassociar($boleto);
                           return true;
                       }
                   } else {
                       // DESASSOCIA APENAS UM BOLETO
                       self::desassociar($boletos);
                       return true;
                   }
               }
               return false;
           } catch (\Exception $e) {
               throw $e;
           }
       }

   }