<?php
/**
 * PercorreReajuste.
 *
 * @package Logic
 *
 * @author  alessandrobartels
 *
 */
class PercorreReajuste{

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_nrEstruturaTrab;
    private $_cdOperador;
    private $_simulacao;
    private $_retrocesso;
    private $_dsOrdem;
    private $_dt_alt_salario;
    private $_mes_competencia;
    /*private $_tipo_calculo;  /* nao usado no codigo */
    private $_calculo_folha;
    private $_tp_calc_mov;
    private $_vinculom;
    private $_pessoa; /* nao usado no codigo */
    private $_dependencia;
    private $_tipo_dependencia;
    private $_nr_tipo_altera;
    private $_dsInicio;
    private $_dsFim;
    private $_vrPercentual;
    private $_formulario;
    private $_nrtipovinc; //Novas adições 18/03/2015 para filtrar os vínculos
    private $_nrvincempreg;
    private $_nrEstrutSind;


    /**
     * Construtor.
     *
     * @param string     $tp_processamento Descricao do tipo de processamento.
     * @param Parametros $parametro        Instancia de Parametros.
     * @param integer    $nrFormulario     Numero do Formulario.
     *
     * @author alessandrobartels
     *         alessandrobartels - 11/07/2011 - Implementacao da Organizacao Padrao.
     */
    public function __construct($tp_processamento, Parametros $parametro, $nrFormulario){

        $this->_nrOrgPadrao = $parametro->getNrOrgPadrao();
        $this->_nrOrg = $parametro->getNrOrgTrabalho();
        $this->_cdOperador = $parametro->getCdOperador();
        $this->_nrEstruturaTrab = $parametro->getNrEstrutTrab($this->_nrOrg, $this->_cdOperador);
        $this->_mes_competencia = $parametro->getMesCompetencia();

        // busca os dados do formulario.
        $this->_buscaDadosFormulario($nrFormulario);

        // $this->_dt_alt_salario =  new TkDateTime($dt_alt_salario, TkDateTime::FORMAT_DATE);.
        $this->_retrocesso = false;

        /* Tipo processamento */
        $tp_processamento = strtoupper($tp_processamento);
        switch ($tp_processamento){
        case Parametros::PROC_SIMULACAO :
            $this->_simulacao = true; break;
        case Parametros::PROC_CALCULO :
            $this->_simulacao = false; break;
        case Parametros::PROC_RETROCESSO :
            $this->_retrocesso = true; break;
        default:
            tk_message('Tipo de processo '.$tp_processamento.' inválido', 'warning');
        }

        Inicializa::Container($this->_nrOrg, $this->_nrOrgPadrao, $this->_mes_competencia);
        $this->_vinculom = TkDI::getContainer()->vinculom;
        $this->_pessoa = TkDI::getContainer()->pessoa;

    }


    /**
     * Percorre os vinculos afetuando o reajuste.
     *
     * @throws Exception Nao realiazou o reajuste.
     * @return DemonstrativoAlteracaoSalarial|NULL
     *
     * @author alessandrobartels
     *         alessandrobartels - 11/07/2011 - Implementacao da Organizacao Padrao.
     */
    public function percorre(){

        $tipoAlteracao = new GpetipoalteraEntity();
        $objTipoAlteracao = $tipoAlteracao->retornaTipoAltera(
            $this->_nrOrgPadrao, $this->_nrOrg, $this->_nr_tipo_altera
        );

        $demonstrativoAltSalario = new DemonstrativoAlteracaoSalarial(
            $objTipoAlteracao->getNmtipoaltera(), $this->_mes_competencia->toString(TkDateTime::FORMAT_DATE),
            $this->_vrPercentual
        );

        $reajuste = new Reajuste(
            $this->_nrOrgPadrao, $this->_nrOrg, $this->_mes_competencia, $this->_simulacao, $this->_formulario
        );

        $vinculos = $this->_vinculom->retornaVinculosAtivosMesCompetencia(
            $this->_nrOrg, $this->_mes_competencia, $this->_dsInicio, $this->_dsFim, $this->_dsOrdem, false,
            $this->_nrEstruturaTrab, true, false, $this->_nrtipovinc, $this->_nrvincempreg, $this->_nrEstrutSind
        );

        $qtdVinculos = count($vinculos);

        if ($qtdVinculos > 0) {
            foreach ($vinculos as $vinculo) {
                try {
                    if ($this->_retrocesso === false) {
                        $result_parcial = $reajuste->reajustar($vinculo, $demonstrativoAltSalario, $this->_mes_competencia);
                    } else {
                        $reajuste->retrocesso($vinculo);
                    }
                } catch(Exception $e){
                    throw $e;
                }
            }

            return $demonstrativoAltSalario;
        } else {
            return null;
        }
    }


    /**
     * Instancia GpeFormaltsalarEntity e atribui valores a atributos.
     *
     * @param integer $nrFormulario Numero do formulario.
     *
     * @return void
     *
     * @author alessandrobartels 12/12/2011
     */
    private function _buscaDadosFormulario($nrFormulario) {

        $formulario = new GpeFormaltsalarEntity();
        $this->_formulario = $formulario->retornaDadosFormularioAlteracaoSalarial($this->_nrOrg, $nrFormulario);
        if (is_object($this->_formulario) === true) {
            $this->_nr_tipo_altera = $this->_formulario->getNrtipoaltera();
            $this->_dsInicio = $this->_formulario->getDsinicio();
            $this->_dsFim = $this->_formulario->getDsfim();
            $this->_vrPercentual = $this->_formulario->getVrpercaltera();
            $this->_dsOrdem = $this->_formulario->getDsordem();
            $this->_nrtipovinc = $this->_formulario->getNrtipovinc();
            $this->_nrvincempreg = $this->_formulario->getNrvincempreg();
            $this->_nrEstrutSind = $this->_formulario->getNrestrutsind();
        }
    }


}